from __future__ import absolute_import

from django.apps import AppConfig

class Config(AppConfig):
    name = "sentry_wxwork"

    def ready(self):
        from sentry.auth import register

        from .provider import WxWorkAuthProvider

        register("wxwork", WxWorkAuthProvider)



from __future__ import absolute_import, print_function

from django.conf import settings

API_BASE_URL = 'https://qyapi.weixin.qq.com/cgi-bin'
AUTHORIZE_URL = 'https://open.weixin.qq.com/connect/oauth2/authorize'
QRLOGIN_URL = 'https://open.work.weixin.qq.com/wwopen/sso/qrConnect'
ACCESS_TOKEN_URL = '%s/gettoken' % API_BASE_URL
SCOPE = 'snsapi_base'

CLIENT_ID = getattr(settings, 'WXWORK_CORP_ID', None)
CLIENT_SECRET = getattr(settings, 'WXWORK_SECRET', None)
AGENT_ID = getattr(settings, 'WXWORK_AGENT_ID', None)


# coding: utf-8
"""
Plugin for Sentry which allows SSO Login via WeChat Work.
"""
from __future__ import absolute_import

from django import VERSION as DJANGO_VERSION

if DJANGO_VERSION >= (1, 7):
	default_app_config = "auth_wxwork.apps.Config"
else:
	from sentry.auth import register
	from .provider import WxWorkAuthProvider

	register('wxwork', WxWorkAuthProvider)



from __future__ import absolute_import

import logging
from uuid import uuid4
from six.moves.urllib.parse import urlencode
from sentry.auth.view import AuthView
from sentry.http import safe_urlopen
from sentry.utils import json
from sentry.utils.http import absolute_uri

from .constants import (
    API_BASE_URL, AUTHORIZE_URL, QRLOGIN_URL, ACCESS_TOKEN_URL, CLIENT_ID, CLIENT_SECRET, AGENT_ID, SCOPE
)

ERR_INVALID_STATE = "An error occurred while validating your request."

class WxWorkLogin(AuthView):
    authorize_url = None
    qrlogin_url = None
    client_id = None
    agent_id = None
    scope = ""

    def __init__(self, *args, **kwargs):
        super(WxWorkLogin, self).__init__(*args, **kwargs)
        self.authorize_url = AUTHORIZE_URL
        self.qrlogin_url = QRLOGIN_URL
        self.client_id = CLIENT_ID
        self.agent_id = AGENT_ID
        self.scope = SCOPE

    def get_scope(self):
        return self.scope

    def get_authorize_params(self, state, redirect_uri):
        return {
            "appid": self.client_id,
            "response_type": "code",
            "scope": self.get_scope(),
            "state": state,
            "redirect_uri": redirect_uri,
        }

    def get_qrlogin_params(self, state, redirect_uri):
        return {
            "appid": self.client_id,
            "agentid": self.agent_id,
            "state": state,
            "redirect_uri": redirect_uri,
        }

    def dispatch(self, request, helper):
        if "code" in request.GET:
            return helper.next_step()

        state = uuid4().hex

        if 'wxwork' in request.META.get("HTTP_USER_AGENT"):
            params = self.get_authorize_params(
                state=state, redirect_uri=absolute_uri(helper.get_redirect_url())
            )
            redirect_uri = u"{}?{}#wechat_redirect".format(self.authorize_url, urlencode(params))
        else:
            params = self.get_qrlogin_params(
                state=state, redirect_uri=absolute_uri(helper.get_redirect_url())
            )
            redirect_uri = u"{}?{}".format(self.qrlogin_url, urlencode(params))

        helper.bind_state("state", state)

        return self.redirect(redirect_uri)

class WxWorkCallback(AuthView):
    logger = logging.getLogger('auth_wxwork')

    access_token_url = None
    client_id = None
    client_secret = None

    def __init__(self, *args, **kwargs):
        super(WxWorkCallback, self).__init__(*args, **kwargs)
        self.access_token_url = ACCESS_TOKEN_URL
        self.client_id = CLIENT_ID
        self.client_secret = CLIENT_SECRET

    def exchange_token(self, request, helper, code):
        url = '%s?corpid=%s&corpsecret=%s' % (self.access_token_url, self.client_id, self.client_secret)
        response = safe_urlopen(url)
        self.logger.debug('Response code: %s, content: %s' % (response.status_code, response.content))
        return json.loads(response.content)

    def dispatch(self, request, helper):
        error = request.GET.get("error")
        state = request.GET.get("state")
        code = request.GET.get("code")

        if error:
            return helper.error(error)

        if state != helper.fetch_state("state"):
            return helper.error(ERR_INVALID_STATE)

        data = self.exchange_token(request, helper, code)

        if data['errcode'] != 0:
            return helper.error(data["errmsg"])

        helper.bind_state("code", code)
        helper.bind_state("data", data)

        return helper.next_step()

class FetchUser(AuthView):
    logger = logging.getLogger('auth_wxwork')

    def get_user_id(self, access_token, code):
        url = '%s/user/getuserinfo?access_token=%s&code=%s' %(API_BASE_URL, access_token, code)
        response = safe_urlopen(url)
        self.logger.debug('Response code: %s, content: %s' % (response.status_code, response.content))
        data = json.loads(response.content)
        return data['UserId']

    def get_user_data(self, access_token, user_id):
        url = '%s/user/get?access_token=%s&userid=%s' %(API_BASE_URL, access_token, user_id)
        response = safe_urlopen(url)
        self.logger.debug('Response code: %s, content: %s' % (response.status_code, response.content))
        return json.loads(response.content)

    def handle(self, request, helper):
        data = helper.fetch_state("data")
        code = helper.fetch_state("code")
        
        user_id = self.get_user_id(data['access_token'], code)
        user = self.get_user_data(data['access_token'], user_id)

        helper.bind_state('user', user)
        return helper.next_step()


#!/usr/bin/env python
# coding: utf-8
from setuptools import setup

from sentry_wxwork import __version__

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='sentry_wxwork',
    version=__version__,
    packages=['sentry_wxwork', 'auth_wxwork'],
    url='https://github.com/tsl0922/sentry-wxwork',
    author='Shuanglei Tao',
    author_email='tsl0922@gmail.com',
    description='Plugin for Sentry which integrates with WeChat Work.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    entry_points={
        'sentry.apps': [
            'auth_wxwork = auth_wxwork',
         ],
        'sentry.plugins': [
            'sentry_wxwork = sentry_wxwork.plugin:WxworkNotificationsPlugin',
        ],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Bug Tracking',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: System :: Monitoring',
    ],
    include_package_data=True,
)


# coding: utf-8
import logging
from datetime import datetime, timedelta
from collections import defaultdict

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from sentry.plugins.bases import notify
from sentry.http import safe_urlopen
from sentry.utils import json
from sentry.utils.safe import safe_execute
from sentry.utils.forms import form_to_config
from sentry.integrations import FeatureDescription, IntegrationFeatures
from sentry.exceptions import PluginError

from . import __version__, __doc__ as package_doc


class WxworkNotificationsOptionsForm(notify.NotificationConfigurationForm):
    api_origin = forms.CharField(
        label=_('API origin'),
        widget=forms.TextInput(attrs={'placeholder': 'https://qyapi.weixin.qq.com/cgi-bin'}),
        initial='https://qyapi.weixin.qq.com/cgi-bin'
    )
    api_secret = forms.CharField(
        label=_('API secret'),
        widget=forms.PasswordInput(attrs={'placeholder': 'vQT_03RDVA3uE6JDASDASDAiXUvccqV8mDgLdLI'}),
        initial=getattr(settings, 'WXWORK_SECRET', None)
    )
    corp_id = forms.CharField(
        label=_('Corp ID'),
        widget=forms.TextInput(attrs={'placeholder': 'wwabcddzxdkrsdv'}),
        initial=getattr(settings, 'WXWORK_CORP_ID', None)
    )
    agent_id = forms.CharField(
        label=_('Agent ID'),
        widget=forms.TextInput(attrs={'placeholder': '1'}),
        initial=getattr(settings, 'WXWORK_AGENT_ID', None)
    )
    to_user = forms.CharField(
        label=_('Receivers: user list'),
        widget=forms.TextInput(attrs={'placeholder': 'UserID1|UserID2|UserID3'}),
        help_text=_('NOTE: user, party, tag list can not be empty at the same time'),
        required=False
    )
    to_party = forms.CharField(
        label=_('Receivers: party list'),
        widget=forms.TextInput(attrs={'placeholder': 'PartyID1|PartyID2'}),
        help_text=_('NOTE: user, party, tag list can not be empty at the same time'),
        required=False
    )
    to_tag = forms.CharField(
        label=_('Receivers: tag list'),
        widget=forms.TextInput(attrs={'placeholder': 'TagID1 | TagID2'}),
        help_text=_('NOTE: user, party, tag list can not be empty at the same time'),
        required=False
    )
    message_template = forms.CharField(
        label=_('Message template'),
        widget=forms.Textarea(attrs={'class': 'span4'}),
        help_text=_('Set in standard python\'s {}-format convention, available names are: '
                    '{project_name}, {url}, {title}, {message}, {tag[%your_tag%]}'),
        initial='**[{project_name}]** [{tag[level]}: {title}]({url})\n\n> {message}'
    )

class WxworkNotificationsPlugin(notify.NotificationPlugin):
    title = 'WeChat Work'
    slug = 'sentry_wxwork'
    description = package_doc
    version = __version__
    author = 'Shuanglei Tao'
    author_url = 'https://github.com/tsl0922/sentry-wxwork'
    resource_links = [
        ('Bug Tracker', 'https://github.com/tsl0922/sentry-wxwork/issues'),
        ('Source', 'https://github.com/tsl0922/sentry-wxwork'),
        ('Reference', 'https://work.weixin.qq.com/api/doc/90000/90135/90664'),
    ]

    conf_key = 'sentry_wxwork'
    conf_title = title
    project_conf_form = WxworkNotificationsOptionsForm
    logger = logging.getLogger('sentry_wxwork')
    feature_descriptions = [
        FeatureDescription(
            """
            Send notification via WeChat Work for Sentry.
            """,
            IntegrationFeatures.ALERT_RULE,
        )
    ]

    access_token = None

    def is_configured(self, project, **kwargs):
        return bool(self.get_option('api_secret', project) and self.get_option('corp_id', project) and self.get_option('agent_id', project))

    def get_config(self, project, **kwargs):
        form = self.project_conf_form
        if not form:
            return []

        return form_to_config(form)

    def request_token(self, api_origin, api_secret, corp_id):
        url = '%s/gettoken?corpid=%s&corpsecret=%s' % (api_origin, corp_id, api_secret)
        response = safe_urlopen(url)
        self.logger.debug('Response code: %s, content: %s' % (response.status_code, response.content))
        return json.loads(response.content)

    def get_token(self, api_origin, api_secret, corp_id):
        if (not self.access_token) or self.access_token['expires'] < datetime.now():
            data = self.request_token(api_origin, api_secret, corp_id)
            if data['errcode'] != 0:
                raise PluginError("invalid wechat token response: %s" % data)
            self.access_token = {
                'token': data['access_token'],
                'expires': datetime.now() + timedelta(seconds = data['expires_in'])
            }
        return self.access_token['token']

    def build_message(self, group, event):
        the_tags = defaultdict(lambda: '[NA]')
        the_tags.update({k:v for k, v in event.tags})
        names = {
            'title': event.title,
            'tag': the_tags,
            'message': event.message,
            'project_name': group.project.name,
            'url': group.get_absolute_url(),
        }

        template = self.get_option('message_template', group.project)

        text = template.format(**names)
        if len(text) > 2048:
            text = text[:2048]

        return {
            'msgtype': 'markdown',
            'agentid': self.get_option('agent_id', group.project),
            'markdown': {
                'content': text
            }
        }

    def build_url(self, project):
        api_origin = self.get_option('api_origin', project)
        api_secret = self.get_option('api_secret', project)
        corp_id = self.get_option('corp_id', project)
        
        token = self.get_token(api_origin, api_secret, corp_id)

        return '%s/message/send?access_token=%s' % (api_origin, token)

    def send_message(self, payload, project):
        to_user = self.get_option('to_user', project)
        to_party = self.get_option('to_party', project)
        to_tag = self.get_option('to_tag', project)

        if to_user:
            payload['touser'] = to_user
        if to_party:
            payload['toparty'] = to_party
        if to_tag:
            payload['totag'] = to_tag

        self.logger.debug('Sending message to user: %s, party: %s, tag: %s ' % (to_user, to_party, to_tag))
        response = safe_urlopen(method='POST', url=self.build_url(project), json=payload)
        self.logger.debug('Response code: %s, content: %s' % (response.status_code, response.content))

        data = json.loads(response.content)
        if data['errcode'] == 40014 or data['errcode'] == 42001: # access token invalid or expired, retry
            self.access_token = None
            safe_urlopen(method='POST', url=self.build_url(project), json=payload)

    def notify_users(self, group, event, fail_silently=False, **kwargs):
        self.logger.debug('Received notification for event: %s' % event)

        payload = self.build_message(group, event)
        self.logger.debug('Built payload: %s' % payload)
        
        safe_execute(self.send_message, payload, group.project, _with_transaction=False)



from __future__ import absolute_import

import logging
from time import time
from sentry.auth.provider import Provider
from sentry.http import safe_urlopen
from sentry.utils import json
from sentry.auth.exceptions import IdentityNotValid

from .views import WxWorkLogin, WxWorkCallback, FetchUser

from .constants import (
    ACCESS_TOKEN_URL, CLIENT_ID, CLIENT_SECRET
)

class WxWorkAuthProvider(Provider):
    name = 'WeChat Work'

    logger = logging.getLogger('auth_wxwork')

    client_id = CLIENT_ID
    client_secret = CLIENT_SECRET
    access_token_url = ACCESS_TOKEN_URL

    def get_auth_pipeline(self):
        return [
            WxWorkLogin(),
            WxWorkCallback(),
            FetchUser()
        ]

    def build_config(self, config):
        return {}

    def get_identity_data(self, payload):
        return {
            'access_token': payload['access_token'],
            'expires': int(time()) + int(payload['expires_in']),
        }

    def build_identity(self, state):
        data = state['data']
        user_data = state['user']
        return {
            'id': user_data['userid'],
            'email': user_data['email'],
            'name': user_data['name'],
            'data': self.get_identity_data(data),
        }

    def update_identity(self, new_data, current_data):
        return new_data

    def refresh_identity(self, auth_identity):
        url = '%s?corpid=%s&corpsecret=%s' % (self.access_token_url, self.client_id, self.client_secret)
        response = safe_urlopen(url)
        self.logger.debug('Response code: %s, content: %s' % (response.status_code, response.content))
        data = json.loads(response.content)

        if data['errcode'] != 0:
            raise IdentityNotValid('errcode: %d, errmsg: %s' & (data['errcode'], data['errmsg']))

        auth_identity.data.update(self.get_identity_data(data))
        auth_identity.update(data=auth_identity.data)
        